import pygame

class Sign():
    """ Traffic Sign Interface
        Defines the interface for traffic signs which have different
        behaviors in handling cars
    """
    def __init__(self):
        """ Constructor that does nothing """
        pass

    def update(self):
        """ Update the traffic sign's status """
        pass
    

class TrafficLight(Sign):
    """ Traffic Light
        Alternate between red and green depending on the speficied time parameters
        Vehicles may proceed on green and wait on red
    """
    def __init__(self, redtime=None, greentime=None, signLocation = None, redLight=True):
        Sign.__init__(self)

        # Internal properties
        self.trafficClock = 0                        # traffic light clock
        self.clock = pygame.time.Clock()             # general purpose clock
        self.redTime = redtime                       # red light time
        self.greenTime = greentime                   # green light time
        self.redLight = redLight                     # true if red light is on
        self.signLocation = signLocation # signLocation[0]
        self.waitClear = False                       # Wait until the intersection is clear

        self.clock.tick()                            # Start clock

    def getWaitClear(self):
        """ Get wait clear time """
        return self.waitClear
        
    def setWaitClear(self, wait):
        """ Set wait clear to the specified parameter """
        self.waitClear = wait

    def handleVehicle(self, vehicle):
        """ Vehicle stops when traffic light is red, otherwise, it may proceed. """
        print "handleVehicle"
        if( self.redLight ):
            vehicle.setWaiting(True)
        else:
            vehicle.setWaiting(False)

    def update(self):
        """ Update its light status depending of the green and red timer """
        if(self.redLight):
            self.trafficClock += self.clock.tick()
            if( ( self.trafficClock / 1000.0 ) > self.redTime ):
                #print "CHANGE TO GREEN!!!"
                self.redLight = False
                self.waitClear = True
                # add: set image to green light
                self.trafficClock = 0
        else:
            self.trafficClock += self.clock.tick()
            if( ( self.trafficClock / 1000.0 ) > self.greenTime ):
                #print "CHANGE TO RED!!!"
                self.redLight = True
                # add: set image to red light
                self.trafficClock = 0

    def setRedTime(self, redTime):
        """ Red light time setter """
        self.redTime = redTime

    def setGreenTime(self, greenTime):
        """ Green light time setter """
        self.greenTime = greenTime

    def getRedTime(self):
        """ Red light time getter """
        return self.redtime

    def getGreenTime(self):
        """ Green light time getter """
        return self.greentime

    def isRed(self):
        """ Returns true if red light is on """
        return self.redLight
    

class StopSign(Sign):
    """ Stop sign handles vehicles such that they will stop for 2 seconds
        as they enter the cell containing the stop sign. In addition, cars
        from different direction will take turn in proceeding based on their
        order of arrival
    """
    def __init__(self, signLocation = None, stopBlock = None):
        """ Initialize the stop sign with default parameters.
            Initialy it has no car, so no stop time has elapsed. It's rank
            (priority on proceeding) should be 3.
        """
        Sign.__init__(self)
        self.signLocation = signLocation
        self.clock = pygame.time.Clock()
        self.stopSignClock = 0.0
        self.hasCar = False
        self.elapsed = False
        self.rank = 3
        self.stopBlock = stopBlock

    def setRank(self):
        """ Set the rank for this stop sign. Stop sign with lower rank
            (ie rank 0) will have its vehicle move first.
        """
        for ic in self.stopBlock.intersectioncells:
            if ( not ic.sign == None ):
                if(ic.sign.rank == 0):
                    pass
                else:
                    #print "SET RANK = 0"
                    self.rank = 0
                    return
        for ic in self.stopBlock.intersectioncells:
            if ( not ic.sign == None ):
                if(ic.sign.rank == 1):
                    pass
                else:
                    #print "SET RANK = 1"
                    self.rank = 1
                    return
        for ic in self.stopBlock.intersectioncells:
            if ( not ic.sign == None ):
                if(ic.sign.rank == 2):
                    pass
                else:
                    #print "SET RANK = 2"
                    self.rank = 2
                    return
        #print "SET RANK = 3"
        self.rank = 3

    def carEnter(self):
        """ A car approach the stop sign. Set stop sign's
            hasCar to "True". Start counting how long
            the car has been at the stop sign.
        """
        self.hasCar = True
        self.clock.tick()

    def update(self):
        """ Update stop sign's properties. If it has a car, it will
            count how long the car has been there. And set elapsed
            to true if car has waited for 1 second.
        """
        # if there is a car at the stop sign
        if( self.hasCar ):
            self.stopSignClock += self.clock.tick() # keep clock running
            #print self.stopSignClock
            if( self.stopSignClock > 1000.0 ):
                self.elapsed = True # 1 secs has elapsed
                self.stopSignClock = 0.0
